@@ -1,11 +1,11 @@
 ;; -*- no-byte-compile: t; lexical-binding: nil -*-
-(define-package "auto-complete" "1.5.1"
+(define-package "auto-complete" "20251123.1919"
   "Auto Completion for GNU Emacs."
-  '((popup  "0.5.0")
-    (cl-lib "0.5"))
+  '((emacs "25.1")
+    (popup "0.5.8"))
   :url "https://github.com/auto-complete/auto-complete"
-  :commit "0655b7f1e6c0f8475adc55f2b86404a877f26a77"
-  :revdesc "v1.5.1-0-g0655b7f1e6c0"
+  :commit "1dbfb343412f9444dcd2d9a08b7b42c7270c9547"
+  :revdesc "1dbfb343412f"
   :keywords '("completion" "convenience")
   :authors '(("Tomohiro Matsuyama" . "m2ym.pub@gmail.com"))
-  :maintainers '(("Tomohiro Matsuyama" . "m2ym.pub@gmail.com")))
+  :maintainers '(("Jen-Chieh Shen" . "jcs090218@gmail.com")))
